/*
    This file is part of the kholidays library.

    SPDX-FileCopyrightText: 2021 Laurent Montel <montel@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/
#pragma once
static const double PI = 3.14159265358979323846;
